@@ -21,19 +21,24 @@ from .util import (
     Serializable,
     parse_date,
     altz_to_utctz_str,
-    parse_actor_and_date
+    parse_actor_and_date,
+    from_timestamp,
 )
 from git.compat import text_type
 
 from time import (
     time,
-    altzone
+    daylight,
+    altzone,
+    timezone,
+    localtime
 )
 import os
 from io import BytesIO
 import logging
 
 log = logging.getLogger('git.objects.commit')
+log.addHandler(logging.NullHandler())
 
 __all__ = ('Commit', )
 
@@ -140,6 +145,14 @@ class Commit(base.Object, Iterable, Diffable, Traversable, Serializable):
             super(Commit, self)._set_cache_(attr)
         # END handle attrs
 
+    @property
+    def authored_datetime(self):
+        return from_timestamp(self.authored_date, self.author_tz_offset)
+
+    @property
+    def committed_datetime(self):
+        return from_timestamp(self.committed_date, self.committer_tz_offset)
+
     @property
     def summary(self):
         """:return: First line of the commit message"""
@@ -265,7 +278,8 @@ class Commit(base.Object, Iterable, Diffable, Traversable, Serializable):
             finalize_process(proc_or_stream)
 
     @classmethod
-    def create_from_tree(cls, repo, tree, message, parent_commits=None, head=False, author=None, committer=None):
+    def create_from_tree(cls, repo, tree, message, parent_commits=None, head=False, author=None, committer=None,
+                         author_date=None, commit_date=None):
         """Commit the given tree, creating a commit object.
 
         :param repo: Repo object the commit should be part of
@@ -287,6 +301,8 @@ class Commit(base.Object, Iterable, Diffable, Traversable, Serializable):
             configuration is used to obtain this value.
         :param committer: The name of the committer, optional. If unset, the
             repository configuration is used to obtain this value.
+        :param author_date: The timestamp for the author field
+        :param commit_date: The timestamp for the committer field
 
         :return: Commit object representing the new commit
 
@@ -323,17 +339,22 @@ class Commit(base.Object, Iterable, Diffable, Traversable, Serializable):
 
         # PARSE THE DATES
         unix_time = int(time())
-        offset = altzone
+        is_dst = daylight and localtime().tm_isdst > 0
+        offset = altzone if is_dst else timezone
 
         author_date_str = env.get(cls.env_author_date, '')
-        if author_date_str:
+        if author_date:
+            author_time, author_offset = parse_date(author_date)
+        elif author_date_str:
             author_time, author_offset = parse_date(author_date_str)
         else:
             author_time, author_offset = unix_time, offset
         # END set author time
 
         committer_date_str = env.get(cls.env_committer_date, '')
-        if committer_date_str:
+        if commit_date:
+            committer_time, committer_offset = parse_date(commit_date)
+        elif committer_date_str:
             committer_time, committer_offset = parse_date(committer_date_str)
         else:
             committer_time, committer_offset = unix_time, offset
@@ -445,7 +466,7 @@ class Commit(base.Object, Iterable, Diffable, Traversable, Serializable):
         next_line = readline()
         while next_line.startswith(b'mergetag '):
             next_line = readline()
-            while next_line.startswith(' '):
+            while next_line.startswith(b' '):
                 next_line = readline()
         # end skip mergetags
 
@@ -480,14 +501,14 @@ class Commit(base.Object, Iterable, Diffable, Traversable, Serializable):
 
         try:
             self.author, self.authored_date, self.author_tz_offset = \
-                parse_actor_and_date(author_line.decode(self.encoding))
+                parse_actor_and_date(author_line.decode(self.encoding, 'replace'))
         except UnicodeDecodeError:
             log.error("Failed to decode author line '%s' using encoding %s", author_line, self.encoding,
                       exc_info=True)
 
         try:
             self.committer, self.committed_date, self.committer_tz_offset = \
-                parse_actor_and_date(committer_line.decode(self.encoding))
+                parse_actor_and_date(committer_line.decode(self.encoding, 'replace'))
         except UnicodeDecodeError:
             log.error("Failed to decode committer line '%s' using encoding %s", committer_line, self.encoding,
                       exc_info=True)
@@ -497,7 +518,7 @@ class Commit(base.Object, Iterable, Diffable, Traversable, Serializable):
         # The end of our message stream is marked with a newline that we strip
         self.message = stream.read()
         try:
-            self.message = self.message.decode(self.encoding)
+            self.message = self.message.decode(self.encoding, 'replace')
         except UnicodeDecodeError:
             log.error("Failed to decode message '%s' using encoding %s", self.message, self.encoding, exc_info=True)
         # END exception handling
