@@ -32,13 +32,11 @@ from git.index import IndexFile
 from git.config import GitConfigParser
 from git.remote import (
     Remote,
-    add_progress
+    add_progress,
+    to_progress_instance
 )
 
-from git.db import (
-    GitCmdObjectDB,
-    GitDB
-)
+from git.db import GitCmdObjectDB
 
 from gitdb.util import (
     join,
@@ -54,18 +52,24 @@ from .fun import (
 )
 from git.compat import (
     text_type,
-    defenc
+    defenc,
+    PY3,
+    safe_decode,
+    range,
 )
 
 import os
 import sys
 import re
+from collections import namedtuple
 
-DefaultDBType = GitDB
+DefaultDBType = GitCmdObjectDB
 if sys.version_info[:2] < (2, 5):     # python 2.4 compatiblity
     DefaultDBType = GitCmdObjectDB
 # END handle python 2.4
 
+BlameEntry = namedtuple('BlameEntry', ['commit', 'linenos', 'orig_path', 'orig_linenos'])
+
 
 __all__ = ('Repo', )
 
@@ -251,7 +255,9 @@ class Repo(object):
 
     @property
     def index(self):
-        """:return: IndexFile representing this repository's index."""
+        """:return: IndexFile representing this repository's index.
+        :note: This property can be expensive, as the returned ``IndexFile`` will be
+         reinitialized. It's recommended to re-use the object."""
         return IndexFile(self)
 
     @property
@@ -405,7 +411,7 @@ class Repo(object):
         """
         :return:
             GitConfigParser allowing to write values of the specified configuration file level.
-            Config writers should be retrieved, used to change the configuration ,and written
+            Config writers should be retrieved, used to change the configuration, and written
             right away as they will lock the configuration file in question and prevent other's
             to write it.
 
@@ -464,7 +470,7 @@ class Repo(object):
             max_count and skip
 
         :note: to receive only commits between two named revisions, use the
-            "revA..revB" revision specifier
+            "revA...revB" revision specifier
 
         :return ``git.Commit[]``"""
         if rev is None:
@@ -503,6 +509,21 @@ class Repo(object):
 
         return res
 
+    def is_ancestor(self, ancestor_rev, rev):
+        """Check if a commit  is an ancestor of another
+
+        :param ancestor_rev: Rev which should be an ancestor
+        :param rev: Rev to test against ancestor_rev
+        :return: ``True``, ancestor_rev is an accestor to rev.
+        """
+        try:
+            self.git.merge_base(ancestor_rev, rev, is_ancestor=True)
+        except GitCommandError as err:
+            if err.status == 1:
+                return False
+            raise
+        return True
+
     def _get_daemon_export(self):
         filename = join(self.git_dir, self.DAEMON_EXPORT_FILE)
         return os.path.exists(filename)
@@ -563,7 +584,8 @@ class Repo(object):
     alternates = property(_get_alternates, _set_alternates,
                           doc="Retrieve a list of alternates paths or set a list paths to be used as alternates")
 
-    def is_dirty(self, index=True, working_tree=True, untracked_files=False):
+    def is_dirty(self, index=True, working_tree=True, untracked_files=False,
+                 submodules=True, path=None):
         """
         :return:
             ``True``, the repository is considered dirty. By default it will react
@@ -575,7 +597,11 @@ class Repo(object):
             return False
 
         # start from the one which is fastest to evaluate
-        default_args = ('--abbrev=40', '--full-index', '--raw')
+        default_args = ['--abbrev=40', '--full-index', '--raw']
+        if not submodules:
+            default_args.append('--ignore-submodules')
+        if path:
+            default_args.append(path)
         if index:
             # diff index against HEAD
             if isfile(self.index.path) and \
@@ -588,7 +614,7 @@ class Repo(object):
                 return True
         # END working tree handling
         if untracked_files:
-            if len(self.untracked_files):
+            if len(self._get_untracked_files(path, ignore_submodules=not submodules)):
                 return True
         # END untracked files
         return False
@@ -603,11 +629,19 @@ class Repo(object):
             are relative to the current working directory of the git command.
 
         :note:
-            ignored files will not appear here, i.e. files mentioned in .gitignore"""
+            ignored files will not appear here, i.e. files mentioned in .gitignore
+        :note:
+            This property is expensive, as no cache is involved. To process the result, please
+            consider caching it yourself."""
+        return self._get_untracked_files()
+
+    def _get_untracked_files(self, *args, **kwargs):
         # make sure we get all files, no only untracked directores
-        proc = self.git.status(porcelain=True,
+        proc = self.git.status(*args,
+                               porcelain=True,
                                untracked_files=True,
-                               as_process=True)
+                               as_process=True,
+                               **kwargs)
         # Untracked files preffix in porcelain mode
         prefix = "?? "
         untracked_files = list()
@@ -618,7 +652,12 @@ class Repo(object):
             filename = line[len(prefix):].rstrip('\n')
             # Special characters are escaped
             if filename[0] == filename[-1] == '"':
-                filename = filename[1:-1].decode('string_escape')
+                filename = filename[1:-1]
+                if PY3:
+                    # WHATEVER ... it's a mess, but works for me
+                    filename = filename.encode('ascii').decode('unicode_escape').encode('latin1').decode(defenc)
+                else:
+                    filename = filename.decode('string_escape').decode(defenc)
             untracked_files.append(filename)
         finalize_process(proc)
         return untracked_files
@@ -630,7 +669,69 @@ class Repo(object):
         :return: Head to the active branch"""
         return self.head.reference
 
-    def blame(self, rev, file):
+    def blame_incremental(self, rev, file, **kwargs):
+        """Iterator for blame information for the given file at the given revision.
+
+        Unlike .blame(), this does not return the actual file's contents, only
+        a stream of BlameEntry tuples.
+
+        :parm rev: revision specifier, see git-rev-parse for viable options.
+        :return: lazy iterator of BlameEntry tuples, where the commit
+                 indicates the commit to blame for the line, and range
+                 indicates a span of line numbers in the resulting file.
+
+        If you combine all line number ranges outputted by this command, you
+        should get a continuous range spanning all line numbers in the file.
+        """
+        data = self.git.blame(rev, '--', file, p=True, incremental=True, stdout_as_string=False, **kwargs)
+        commits = dict()
+
+        stream = (line for line in data.split(b'\n') if line)
+        while True:
+            line = next(stream)  # when exhausted, casues a StopIteration, terminating this function
+            hexsha, orig_lineno, lineno, num_lines = line.split()
+            lineno = int(lineno)
+            num_lines = int(num_lines)
+            orig_lineno = int(orig_lineno)
+            if hexsha not in commits:
+                # Now read the next few lines and build up a dict of properties
+                # for this commit
+                props = dict()
+                while True:
+                    line = next(stream)
+                    if line == b'boundary':
+                        # "boundary" indicates a root commit and occurs
+                        # instead of the "previous" tag
+                        continue
+
+                    tag, value = line.split(b' ', 1)
+                    props[tag] = value
+                    if tag == b'filename':
+                        # "filename" formally terminates the entry for --incremental
+                        orig_filename = value
+                        break
+
+                c = Commit(self, hex_to_bin(hexsha),
+                           author=Actor(safe_decode(props[b'author']),
+                                        safe_decode(props[b'author-mail'].lstrip(b'<').rstrip(b'>'))),
+                           authored_date=int(props[b'author-time']),
+                           committer=Actor(safe_decode(props[b'committer']),
+                                           safe_decode(props[b'committer-mail'].lstrip(b'<').rstrip(b'>'))),
+                           committed_date=int(props[b'committer-time']))
+                commits[hexsha] = c
+            else:
+                # Discard the next line (it's a filename end tag)
+                line = next(stream)
+                tag, value = line.split(b' ', 1)
+                assert tag == b'filename', 'Unexpected git blame output'
+                orig_filename = value
+
+            yield BlameEntry(commits[hexsha],
+                             range(lineno, lineno + num_lines),
+                             safe_decode(orig_filename),
+                             range(orig_lineno, orig_lineno + num_lines))
+
+    def blame(self, rev, file, incremental=False, **kwargs):
         """The blame information for the given file at the given revision.
 
         :parm rev: revision specifier, see git-rev-parse for viable options.
@@ -639,7 +740,10 @@ class Repo(object):
             A list of tuples associating a Commit object with a list of lines that
             changed within the given commit. The Commit objects will be given in order
             of appearance."""
-        data = self.git.blame(rev, '--', file, p=True, stdout_as_string=False)
+        if incremental:
+            return self.blame_incremental(rev, file, **kwargs)
+
+        data = self.git.blame(rev, '--', file, p=True, stdout_as_string=False, **kwargs)
         commits = dict()
         blames = list()
         info = None
@@ -713,8 +817,7 @@ class Repo(object):
                                            authored_date=info['author_date'],
                                            committer=Actor._from_string(
                                                info['committer'] + ' ' + info['committer_email']),
-                                           committed_date=info['committer_date'],
-                                           message=info['summary'])
+                                           committed_date=info['committer_date'])
                                 commits[sha] = c
                             # END if commit objects needs initial creation
                             if not is_binary:
@@ -737,7 +840,7 @@ class Repo(object):
         return blames
 
     @classmethod
-    def init(cls, path=None, mkdir=True, **kwargs):
+    def init(cls, path=None, mkdir=True, odbt=DefaultDBType, **kwargs):
         """Initialize a git repository at the given path if specified
 
         :param path:
@@ -750,6 +853,11 @@ class Repo(object):
             already exists. Creates the directory with a mode=0755.
             Only effective if a path is explicitly given
 
+        :param odbt:
+            Object DataBase type - a type which is constructed by providing
+            the directory containing the database objects, i.e. .git/objects.
+            It will be used to access all object data
+
         :parm kwargs:
             keyword arguments serving as additional options to the git-init command
 
@@ -762,10 +870,13 @@ class Repo(object):
         # git command automatically chdir into the directory
         git = Git(path)
         git.init(**kwargs)
-        return cls(path)
+        return cls(path, odbt=odbt)
 
     @classmethod
     def _clone(cls, git, url, path, odb_default_type, progress, **kwargs):
+        if progress is not None:
+            progress = to_progress_instance(progress)
+
         # special handling for windows for path at which the clone should be
         # created.
         # tilde '~' will be expanded to the HOME no matter where the ~ occours. Hence
@@ -797,7 +908,8 @@ class Repo(object):
             if progress:
                 handle_process_output(proc, None, progress.new_message_handler(), finalize_process)
             else:
-                finalize_process(proc)
+                (stdout, stderr) = proc.communicate()
+                finalize_process(proc, stderr=stderr)
             # end handle progress
         finally:
             if prev_cwd is not None:
@@ -839,14 +951,19 @@ class Repo(object):
         return self._clone(self.git, self.git_dir, path, type(self.odb), progress, **kwargs)
 
     @classmethod
-    def clone_from(cls, url, to_path, progress=None, **kwargs):
+    def clone_from(cls, url, to_path, progress=None, env=None, **kwargs):
         """Create a clone from the given URL
+
         :param url: valid git url, see http://www.kernel.org/pub/software/scm/git/docs/git-clone.html#URLS
         :param to_path: Path to which the repository should be cloned to
         :param progress: See 'git.remote.Remote.push'.
+        :param env: Optional dictionary containing the desired environment variables.
         :param kwargs: see the ``clone`` method
         :return: Repo instance pointing to the cloned directory"""
-        return cls._clone(Git(os.getcwd()), url, to_path, GitCmdObjectDB, progress, **kwargs)
+        git = Git(os.getcwd())
+        if env is not None:
+            git.update_environment(**env)
+        return cls._clone(git, url, to_path, GitCmdObjectDB, progress, **kwargs)
 
     def archive(self, ostream, treeish=None, prefix=None, **kwargs):
         """Archive the tree at the given revision.
