@@ -1,11 +1,13 @@
-#-*-coding:utf-8-*-
+# -*- coding: utf-8 -*-
 # test_git.py
 # Copyright (C) 2008, 2009 Michael Trier (mtrier@gmail.com) and contributors
 #
 # This module is part of GitPython and is released under
 # the BSD License: http://www.opensource.org/licenses/bsd-license.php
 import os
+import sys
 import mock
+import subprocess
 
 from git.test.lib import (
     TestBase,
@@ -63,11 +65,15 @@ class TestGit(TestBase):
 
     def test_it_transforms_kwargs_into_git_command_arguments(self):
         assert_equal(["-s"], self.git.transform_kwargs(**{'s': True}))
-        assert_equal(["-s5"], self.git.transform_kwargs(**{'s': 5}))
+        assert_equal(["-s", "5"], self.git.transform_kwargs(**{'s': 5}))
 
         assert_equal(["--max-count"], self.git.transform_kwargs(**{'max_count': True}))
         assert_equal(["--max-count=5"], self.git.transform_kwargs(**{'max_count': 5}))
 
+        # Multiple args are supported by using lists/tuples
+        assert_equal(["-L", "1-3", "-L", "12-18"], self.git.transform_kwargs(**{'L': ('1-3', '12-18')}))
+        assert_equal(["-C", "-C"], self.git.transform_kwargs(**{'C': [True, True]}))
+
         # order is undefined
         res = self.git.transform_kwargs(**{'s': True, 't': True})
         assert ['-s', '-t'] == res or ['-t', '-s'] == res
@@ -192,10 +198,41 @@ class TestGit(TestBase):
         rw_repo = Repo.init(os.path.join(rw_dir, 'repo'))
         remote = rw_repo.create_remote('ssh-origin', "ssh://git@server/foo")
 
-        with rw_repo.git.custom_environment(GIT_SSH=path):
-            try:
-                remote.fetch()
-            except GitCommandError as err:
-                assert 'FOO' in str(err)
+        # This only works if we are not evaluating git-push/pull output in a thread !
+        import select
+        if hasattr(select, 'poll'):
+            with rw_repo.git.custom_environment(GIT_SSH=path):
+                try:
+                    remote.fetch()
+                except GitCommandError as err:
+                    if sys.version_info[0] < 3 and sys.platform == 'darwin':
+                        assert 'ssh-origin' in str(err)
+                        assert err.status == 128
+                    else:
+                        assert 'FOO' in str(err)
+                # end
             # end
-        # end
+        # end if select.poll exists
+
+    def test_handle_process_output(self):
+        from git.cmd import handle_process_output
+
+        line_count = 5002
+        count = [None, 0, 0]
+
+        def counter_stdout(line):
+            count[1] += 1
+
+        def counter_stderr(line):
+            count[2] += 1
+
+        proc = subprocess.Popen([sys.executable, fixture_path('cat_file.py'), str(fixture_path('issue-301_stderr'))],
+                                stdin=None,
+                                stdout=subprocess.PIPE,
+                                stderr=subprocess.PIPE,
+                                shell=False)
+
+        handle_process_output(proc, counter_stdout, counter_stderr, lambda proc: proc.wait())
+
+        assert count[1] == line_count
+        assert count[2] == line_count
