@@ -105,9 +105,11 @@ class TestRefs(TestBase):
             tv = "testopt"
             writer.set_value(tv, 1)
             assert writer.get_value(tv) == 1
-            del(writer)
+            writer.release()
             assert head.config_reader().get_value(tv) == 1
-            head.config_writer().remove_option(tv)
+            writer = head.config_writer()
+            writer.remove_option(tv)
+            writer.release()
 
             # after the clone, we might still have a tracking branch setup
             head.set_tracking_branch(None)
@@ -273,7 +275,7 @@ class TestRefs(TestBase):
         self.failUnlessRaises(TypeError, RemoteReference.create, rw_repo, "some_name")
 
         # tag ref
-        tag_name = "1.0.2"
+        tag_name = "5.0.2"
         light_tag = TagReference.create(rw_repo, tag_name)
         self.failUnlessRaises(GitCommandError, TagReference.create, rw_repo, tag_name)
         light_tag = TagReference.create(rw_repo, tag_name, "HEAD~1", force=True)
