@@ -8,7 +8,9 @@ from git.test.lib import (
     TestBase,
     with_rw_repo,
     with_rw_and_rw_remote_repo,
-    fixture
+    fixture,
+    GIT_DAEMON_PORT,
+    assert_raises
 )
 from git import (
     RemoteProgress,
@@ -62,6 +64,14 @@ class TestRemoteProgress(RemoteProgress):
         op_id = op_code & self.OP_MASK
         assert op_id in (self.COUNTING, self.COMPRESSING, self.WRITING)
 
+        if op_code & self.WRITING > 0:
+            if op_code & self.BEGIN > 0:
+                assert not message, 'should not have message when remote begins writing'
+            elif op_code & self.END > 0:
+                assert message
+                assert not message.startswith(', '), "Sanitize progress messages: '%s'" % message
+                assert not message.endswith(', '), "Sanitize progress messages: '%s'" % message
+
         self._stages_per_op.setdefault(op_id, 0)
         self._stages_per_op[op_id] = self._stages_per_op[op_id] | (op_code & self.STAGE_MASK)
 
@@ -250,7 +260,7 @@ class TestRemote(TestBase):
         # must clone with a local path for the repo implementation not to freak out
         # as it wants local paths only ( which I can understand )
         other_repo = remote_repo.clone(other_repo_dir, shared=False)
-        remote_repo_url = "git://localhost%s" % remote_repo.git_dir
+        remote_repo_url = "git://localhost:%s%s" % (GIT_DAEMON_PORT, remote_repo.git_dir)
 
         # put origin to git-url
         other_origin = other_repo.remotes.origin
@@ -493,6 +503,11 @@ class TestRemote(TestBase):
         fetch_info_line_fmt = "c437ee5deb8d00cf02f03720693e4c802e99f390	not-for-merge	%s '0.3' of "
         fetch_info_line_fmt += "git://github.com/gitpython-developers/GitPython"
         remote_info_line_fmt = "* [new branch]      nomatter     -> %s"
+
+        self.failUnlessRaises(ValueError, FetchInfo._from_line, self.rorepo,
+                              remote_info_line_fmt % "refs/something/branch",
+                              "269c498e56feb93e408ed4558c8138d750de8893\t\t/Users/ben/test/foo\n")
+
         fi = FetchInfo._from_line(self.rorepo,
                                   remote_info_line_fmt % "local/master",
                                   fetch_info_line_fmt % 'remote-tracking branch')
@@ -554,3 +569,47 @@ class TestRemote(TestBase):
         assert res[0].remote_ref_path == 'refs/pull/1/head'
         assert res[0].ref.path == 'refs/heads/pull/1/head'
         assert isinstance(res[0].ref, Head)
+
+    @with_rw_repo('HEAD', bare=False)
+    def test_multiple_urls(self, rw_repo):
+        # test addresses
+        test1 = 'https://github.com/gitpython-developers/GitPython'
+        test2 = 'https://github.com/gitpython-developers/gitdb'
+        test3 = 'https://github.com/gitpython-developers/smmap'
+
+        remote = rw_repo.remotes[0]
+        # Testing setting a single URL
+        remote.set_url(test1)
+        assert list(remote.urls) == [test1]
+
+        # Testing replacing that single URL
+        remote.set_url(test1)
+        assert list(remote.urls) == [test1]
+        # Testing adding new URLs
+        remote.set_url(test2, add=True)
+        assert list(remote.urls) == [test1, test2]
+        remote.set_url(test3, add=True)
+        assert list(remote.urls) == [test1, test2, test3]
+        # Testing removing an URL
+        remote.set_url(test2, delete=True)
+        assert list(remote.urls) == [test1, test3]
+        # Testing changing an URL
+        remote.set_url(test3, test2)
+        assert list(remote.urls) == [test1, test2]
+
+        # will raise: fatal: --add --delete doesn't make sense
+        assert_raises(GitCommandError, remote.set_url, test2, add=True, delete=True)
+
+        # Testing on another remote, with the add/delete URL
+        remote = rw_repo.create_remote('another', url=test1)
+        remote.add_url(test2)
+        assert list(remote.urls) == [test1, test2]
+        remote.add_url(test3)
+        assert list(remote.urls) == [test1, test2, test3]
+        # Testing removing all the URLs
+        remote.delete_url(test2)
+        assert list(remote.urls) == [test1, test3]
+        remote.delete_url(test1)
+        assert list(remote.urls) == [test3]
+        # will raise fatal: Will not delete all non-push URLs
+        assert_raises(GitCommandError, remote.delete_url, test3)
