@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # test_repo.py
 # Copyright (C) 2008, 2009 Michael Trier (mtrier@gmail.com) and contributors
 #
@@ -32,7 +33,9 @@ from git import (
 )
 from git.repo.fun import touch
 from git.util import join_path_native
-from git.exc import BadObject
+from git.exc import (
+    BadObject,
+)
 from gitdb.util import bin_to_hex
 from git.compat import string_types
 from gitdb.test.lib import with_rw_directory
@@ -41,8 +44,21 @@ import os
 import sys
 import tempfile
 import shutil
+import itertools
 from io import BytesIO
 
+from nose import SkipTest
+
+
+def iter_flatten(lol):
+    for items in lol:
+        for item in items:
+            yield item
+
+
+def flatten(lol):
+    return list(iter_flatten(lol))
+
 
 class TestRepo(TestBase):
 
@@ -255,6 +271,24 @@ class TestRepo(TestBase):
         assert self.rorepo.is_dirty() is False
         self.rorepo._bare = orig_val
 
+    @with_rw_repo('HEAD')
+    def test_is_dirty_with_path(self, rwrepo):
+        assert rwrepo.is_dirty(path="git") is False
+
+        with open(os.path.join(rwrepo.working_dir, "git", "util.py"), "at") as f:
+            f.write("junk")
+        assert rwrepo.is_dirty(path="git") is True
+        assert rwrepo.is_dirty(path="doc") is False
+
+        rwrepo.git.add(os.path.join("git", "util.py"))
+        assert rwrepo.is_dirty(index=False, path="git") is False
+        assert rwrepo.is_dirty(path="git") is True
+
+        with open(os.path.join(rwrepo.working_dir, "doc", "no-such-file.txt"), "wt") as f:
+            f.write("junk")
+        assert rwrepo.is_dirty(path="doc") is False
+        assert rwrepo.is_dirty(untracked_files=True, path="doc") is True
+
     def test_head(self):
         assert self.rorepo.head.reference.object == self.rorepo.active_branch.object
 
@@ -291,7 +325,7 @@ class TestRepo(TestBase):
         assert_equal('Tom Preston-Werner', c.committer.name)
         assert_equal('tom@mojombo.com', c.committer.email)
         assert_equal(1191997100, c.committed_date)
-        assert_equal('initial grit setup', c.message)
+        self.assertRaisesRegexp(ValueError, "634396b2f541a9f2d58b00be1a07f0c358b999b3 missing", lambda: c.message)
 
         # test the 'lines per commit' entries
         tlist = b[0][1]
@@ -317,6 +351,27 @@ class TestRepo(TestBase):
         assert c, "Should have executed at least one blame command"
         assert nml, "There should at least be one blame commit that contains multiple lines"
 
+    @patch.object(Git, '_call_process')
+    def test_blame_incremental(self, git):
+        git.return_value = fixture('blame_incremental')
+        blame_output = self.rorepo.blame_incremental('9debf6b0aafb6f7781ea9d1383c86939a1aacde3', 'AUTHORS')
+        blame_output = list(blame_output)
+        assert len(blame_output) == 5
+
+        # Check all outputted line numbers
+        ranges = flatten([entry.linenos for entry in blame_output])
+        assert ranges == flatten([range(2, 3), range(14, 15), range(1, 2), range(3, 14), range(15, 17)]), str(ranges)
+
+        commits = [entry.commit.hexsha[:7] for entry in blame_output]
+        assert commits == ['82b8902', '82b8902', 'c76852d', 'c76852d', 'c76852d'], str(commits)
+
+        # Original filenames
+        assert all([entry.orig_path == u'AUTHORS' for entry in blame_output])
+
+        # Original line numbers
+        orig_ranges = flatten([entry.orig_linenos for entry in blame_output])
+        assert orig_ranges == flatten([range(2, 3), range(14, 15), range(1, 2), range(2, 13), range(13, 15)]), str(orig_ranges)  # noqa
+
     @patch.object(Git, '_call_process')
     def test_blame_complex_revision(self, git):
         git.return_value = fixture('blame_complex_revision')
@@ -324,17 +379,21 @@ class TestRepo(TestBase):
         assert len(res) == 1
         assert len(res[0][1]) == 83, "Unexpected amount of parsed blame lines"
 
-    def test_untracked_files(self):
-        base = self.rorepo.working_tree_dir
-        files = (join_path_native(base, "__test_myfile"),
-                 join_path_native(base, "__test_other_file"))
-        num_recently_untracked = 0
-        try:
+    @with_rw_repo('HEAD', bare=False)
+    def test_untracked_files(self, rwrepo):
+        for (run, repo_add) in enumerate((rwrepo.index.add, rwrepo.git.add)):
+            base = rwrepo.working_tree_dir
+            files = (join_path_native(base, u"%i_test _myfile" % run),
+                     join_path_native(base, "%i_test_other_file" % run),
+                     join_path_native(base, u"%i__çava verböten" % run),
+                     join_path_native(base, u"%i_çava-----verböten" % run))
+
+            num_recently_untracked = 0
             for fpath in files:
                 fd = open(fpath, "wb")
                 fd.close()
             # END for each filename
-            untracked_files = self.rorepo.untracked_files
+            untracked_files = rwrepo.untracked_files
             num_recently_untracked = len(untracked_files)
 
             # assure we have all names - they are relative to the git-dir
@@ -342,13 +401,10 @@ class TestRepo(TestBase):
             for utfile in untracked_files:
                 num_test_untracked += join_path_native(base, utfile) in files
             assert len(files) == num_test_untracked
-        finally:
-            for fpath in files:
-                if os.path.isfile(fpath):
-                    os.remove(fpath)
-        # END handle files
 
-        assert len(self.rorepo.untracked_files) == (num_recently_untracked - len(files))
+            repo_add(untracked_files)
+            assert len(rwrepo.untracked_files) == (num_recently_untracked - len(files))
+        # end for each run
 
     def test_config_reader(self):
         reader = self.rorepo.config_reader()                # all config files
@@ -447,7 +503,7 @@ class TestRepo(TestBase):
         assert s.readline() == l1
         assert s.readline() == l2
         assert s.readline() == l3
-        assert s.readline() == ''
+        assert s.readline() == b''
         assert s._stream.tell() == len(d)
 
         # readline limit
@@ -458,13 +514,13 @@ class TestRepo(TestBase):
         # readline on tiny section
         s = mktiny()
         assert s.readline() == l1p
-        assert s.readline() == ''
+        assert s.readline() == b''
         assert s._stream.tell() == ts + 1
 
         # read no limit
         s = mkfull()
         assert s.read() == d[:-1]
-        assert s.read() == ''
+        assert s.read() == b''
         assert s._stream.tell() == len(d)
 
         # read limit
@@ -654,7 +710,7 @@ class TestRepo(TestBase):
         assert rev_parse('@{1}') != head.commit
 
     def test_repo_odbtype(self):
-        target_type = GitDB
+        target_type = GitCmdObjectDB
         if sys.version_info[:2] < (2, 5):
             target_type = GitCmdObjectDB
         assert isinstance(self.rorepo.odb, target_type)
@@ -737,12 +793,23 @@ class TestRepo(TestBase):
         new_file_path = os.path.join(rw_dir, "new_file.ext")
         touch(new_file_path)
         r.index.add([new_file_path])
-        r.index.commit("initial commit")
+        r.index.commit("initial commit\nBAD MESSAGE 1\n")
 
         # Now a branch should be creatable
         nb = r.create_head('foo')
         assert nb.is_valid()
 
+        with open(new_file_path, 'w') as f:
+            f.write('Line 1\n')
+
+        r.index.add([new_file_path])
+        r.index.commit("add line 1\nBAD MESSAGE 2\n")
+
+        with open('%s/.git/logs/refs/heads/master' % (rw_dir,), 'r') as f:
+            contents = f.read()
+
+        assert 'BAD MESSAGE' not in contents, 'log is corrupt'
+
     def test_merge_base(self):
         repo = self.rorepo
         c1 = 'f6aa8d1'
@@ -759,7 +826,37 @@ class TestRepo(TestBase):
         for kw in ('a', 'all'):
             res = repo.merge_base(c1, c2, c3, **{kw: True})
             assert isinstance(res, list) and len(res) == 1
-        # end for eaech keyword signalling all merge-bases to be returned
+        # end for each keyword signalling all merge-bases to be returned
 
         # Test for no merge base - can't do as we have
         self.failUnlessRaises(GitCommandError, repo.merge_base, c1, 'ffffff')
+
+    def test_is_ancestor(self):
+        git = self.rorepo.git
+        if git.version_info[:3] < (1, 8, 0):
+            raise SkipTest("git merge-base --is-ancestor feature unsupported")
+
+        repo = self.rorepo
+        c1 = 'f6aa8d1'
+        c2 = '763ef75'
+        self.assertTrue(repo.is_ancestor(c1, c1))
+        self.assertTrue(repo.is_ancestor("master", "master"))
+        self.assertTrue(repo.is_ancestor(c1, c2))
+        self.assertTrue(repo.is_ancestor(c1, "master"))
+        self.assertFalse(repo.is_ancestor(c2, c1))
+        self.assertFalse(repo.is_ancestor("master", c1))
+        for i, j in itertools.permutations([c1, 'ffffff', ''], r=2):
+            self.assertRaises(GitCommandError, repo.is_ancestor, i, j)
+
+    @with_rw_directory
+    def test_work_tree_unsupported(self, rw_dir):
+        git = Git(rw_dir)
+        if git.version_info[:3] < (2, 5, 1):
+            raise SkipTest("worktree feature unsupported")
+
+        rw_master = self.rorepo.clone(join_path_native(rw_dir, 'master_repo'))
+        rw_master.git.checkout('HEAD~10')
+        worktree_path = join_path_native(rw_dir, 'worktree_repo')
+        rw_master.git.worktree('add', worktree_path, 'master')
+
+        self.failUnlessRaises(InvalidGitRepositoryError, Repo, worktree_path)
