@@ -13,6 +13,7 @@ import shutil
 import platform
 import getpass
 import threading
+import logging
 
 # NOTE:  Some of the unused imports might be used/imported by others.
 # Handle once test-cases are back up and running.
@@ -38,7 +39,7 @@ from gitdb.util import (  # NOQA
 __all__ = ("stream_copy", "join_path", "to_native_path_windows", "to_native_path_linux",
            "join_path_native", "Stats", "IndexFileSHA1Writer", "Iterable", "IterableList",
            "BlockingLockFile", "LockFile", 'Actor', 'get_user_id', 'assure_directory_exists',
-           'RemoteProgress', 'rmtree', 'WaitGroup', 'unbare_repo')
+           'RemoteProgress', 'CallableRemoteProgress', 'rmtree', 'WaitGroup', 'unbare_repo')
 
 #{ Utility Methods
 
@@ -149,9 +150,9 @@ def get_user_id():
     return "%s@%s" % (getpass.getuser(), platform.node())
 
 
-def finalize_process(proc):
+def finalize_process(proc, **kwargs):
     """Wait for the process (clone, fetch, pull or push) and handle its errors accordingly"""
-    proc.wait()
+    proc.wait(**kwargs)
 
 #} END utilities
 
@@ -159,32 +160,48 @@ def finalize_process(proc):
 
 
 class RemoteProgress(object):
-
     """
     Handler providing an interface to parse progress information emitted by git-push
     and git-fetch and to dispatch callbacks allowing subclasses to react to the progress.
     """
-    _num_op_codes = 7
-    BEGIN, END, COUNTING, COMPRESSING, WRITING, RECEIVING, RESOLVING = [1 << x for x in range(_num_op_codes)]
+    _num_op_codes = 9
+    BEGIN, END, COUNTING, COMPRESSING, WRITING, RECEIVING, RESOLVING, FINDING_SOURCES, CHECKING_OUT = \
+        [1 << x for x in range(_num_op_codes)]
     STAGE_MASK = BEGIN | END
     OP_MASK = ~STAGE_MASK
 
-    __slots__ = ("_cur_line", "_seen_ops")
-    re_op_absolute = re.compile("(remote: )?([\w\s]+):\s+()(\d+)()(.*)")
-    re_op_relative = re.compile("(remote: )?([\w\s]+):\s+(\d+)% \((\d+)/(\d+)\)(.*)")
+    DONE_TOKEN = 'done.'
+    TOKEN_SEPARATOR = ', '
+
+    __slots__ = ("_cur_line", "_seen_ops", "_error_lines")
+    re_op_absolute = re.compile(r"(remote: )?([\w\s]+):\s+()(\d+)()(.*)")
+    re_op_relative = re.compile(r"(remote: )?([\w\s]+):\s+(\d+)% \((\d+)/(\d+)\)(.*)")
 
     def __init__(self):
         self._seen_ops = list()
+        self._cur_line = None
+        self._error_lines = []
+
+    def error_lines(self):
+        """Returns all lines that started with error: or fatal:"""
+        return self._error_lines
 
     def _parse_progress_line(self, line):
         """Parse progress information from the given line as retrieved by git-push
-        or git-fetch
+        or git-fetch.
+
+        Lines that seem to contain an error (i.e. start with error: or fatal:) are stored
+        separately and can be queried using `error_lines()`.
 
         :return: list(line, ...) list of lines that could not be processed"""
         # handle
         # Counting objects: 4, done.
         # Compressing objects:  50% (1/2)   \rCompressing objects: 100% (2/2)   \rCompressing objects: 100% (2/2), done.
         self._cur_line = line
+        if len(self._error_lines) > 0 or self._cur_line.startswith(('error:', 'fatal:')):
+            self._error_lines.append(self._cur_line)
+            return []
+
         sub_lines = line.split('\r')
         failed_lines = list()
         for sline in sub_lines:
@@ -227,6 +244,10 @@ class RemoteProgress(object):
                 op_code |= self.RECEIVING
             elif op_name == 'Resolving deltas':
                 op_code |= self.RESOLVING
+            elif op_name == 'Finding sources':
+                op_code |= self.FINDING_SOURCES
+            elif op_name == 'Checking out files':
+                op_code |= self.CHECKING_OUT
             else:
                 # Note: On windows it can happen that partial lines are sent
                 # Hence we get something like "CompreReceiving objects", which is
@@ -235,7 +256,6 @@ class RemoteProgress(object):
                 # to make sure we get informed in case the process spits out new
                 # commands at some point.
                 self.line_dropped(sline)
-                sys.stderr.write("Operation name %r unknown - skipping line '%s'" % (op_name, sline))
                 # Note: Don't add this line to the failed lines, as we have to silently
                 # drop it
                 return failed_lines
@@ -252,11 +272,11 @@ class RemoteProgress(object):
             # END message handling
 
             message = message.strip()
-            done_token = ', done.'
-            if message.endswith(done_token):
+            if message.endswith(self.DONE_TOKEN):
                 op_code |= self.END
-                message = message[:-len(done_token)]
+                message = message[:-len(self.DONE_TOKEN)]
             # END end message handling
+            message = message.strip(self.TOKEN_SEPARATOR)
 
             self.update(op_code,
                         cur_count and float(cur_count),
@@ -304,10 +324,21 @@ class RemoteProgress(object):
 
         You may read the contents of the current line in self._cur_line"""
         pass
+        
+
+class CallableRemoteProgress(RemoteProgress):
+    """An implementation forwarding updates to any callable"""
+    __slots__ = ('_callable')
+    
+    def __init__(self, fn):
+        self._callable = fn
+        super(CallableRemoteProgress, self).__init__()
+
+    def update(self, *args, **kwargs):
+        self._callable(*args, **kwargs)
 
 
 class Actor(object):
-
     """Actors hold information about a person acting on the repository. They
     can be committers and authors or anything with a name and an email as
     mentioned in the git log entries."""
@@ -745,8 +776,17 @@ class WaitGroup(object):
             self.cv.notify_all()
         self.cv.release()
 
-    def wait(self):
+    def wait(self, stderr=b''):
         self.cv.acquire()
         while self.count > 0:
             self.cv.wait()
         self.cv.release()
+
+
+class NullHandler(logging.Handler):
+    def emit(self, record):
+        pass
+
+# In Python 2.6, there is no NullHandler yet. Let's monkey-patch it for a workaround.
+if not hasattr(logging, 'NullHandler'):
+    logging.NullHandler = NullHandler
